import builtins
import datetime
import time

import fastapi
import logging
from conftab.model import get_db, Server, Conf, Session
from conftab.default import SQLALCHEMY_DATABASE_URL

app = fastapi.FastAPI()


@app.get('/')
async def index(req: fastapi.Request):
    return {"server": 'conftab', "msg": "hello! http://127.0.0.1:7788/html/conf", "db": SQLALCHEMY_DATABASE_URL,
            'server_time': time.time()}


@app.get('/api/conf/get')
async def get_conf(req: fastapi.Request, db: Session = fastapi.Depends(get_db)):
    logging.info(f'{dict(req.items())}')
    data = (dict(await req.form()))
    res = db.query(Conf).filter(*(getattr(Conf, k) == v for k, v in data.items())).limit(1).all()
    if len(res) == 1:
        res = res[0]
        return {'data': getattr(builtins, res.value_type)(res.value), "raw": res}
    elif len(res) > 1:
        res_raw = res
        return {'data': None, 'err': '配置超过两个，请检查', "raw": res_raw}
    else:
        return {'data': None, 'raw': {}}


@app.get('/api/conf/list')
async def list_conf(req: fastapi.Request, db: Session = fastapi.Depends(get_db)):
    data = (dict(await req.form()))
    res = db.query(Conf).filter(*(getattr(Conf, k) == v for k, v in data.items())).all()
    return res


@app.post('/api/conf/set')
async def set_conf(req: fastapi.Request, db: Session = fastapi.Depends(get_db)):
    logging.info(f'{dict(req.items())}')
    data = (dict(await req.form()))
    c = Conf(**data)
    c.uuid = f'{c.project}--{c.env}--{c.ver}--{c.key}--{c.value}'
    c.timecreate = time.time()
    c.time_create = datetime.datetime.now()
    db.merge(c)
    db.commit()
    return


def format_to_table(data, keys=None, pk='id'):
    if not data:
        return '表格无数据'
    if not keys:
        keys = set()
        for d in data:
            for k, v in d.items():
                keys.add(k)
        keys = list(keys)
        keys.sort()
        # noinspection PyBroadException
        try:
            keys.remove(pk)
            keys.append(pk)
            keys.reverse()
        except Exception:
            pass
    return """
    <style> 
    table td{border:1px solid #F00} 
    </style>""" + """
    <table>
    <tr>
        {}
    </tr>""".format(
        ''.join(f"<th>{k}</th>" for k in keys)) + ''.join("""
    <tr>
        {}
    </tr>""".format(
        ''.join("<td>{}</td>".format(d.get(k, '/')) for k in keys)) for d in data) + """
    </table>"""


@app.get('/html/conf', response_class=fastapi.responses.HTMLResponse)
async def html_list_conf(req: fastapi.Request, db: Session = fastapi.Depends(get_db)):
    data = (dict(req.query_params))
    res = list(d.to_dict() for d in db.query(Conf).filter(*(getattr(Conf, k) == v for k, v in data.items())).all())
    return format_to_table(
        res,
        keys=['project', 'env', 'ver', 'time_create', 'time_update', 'value_type', 'key', 'value'])



import requests
import builtins
from conftab.default import WEB_HOST, WEB_PORT, PROJECT_NAME, ENV, VERSION


def get_conf(key, default=None, manager_url=f'{WEB_HOST}:{WEB_PORT}', project=PROJECT_NAME, env=ENV, ver=VERSION):
    res = requests.get(f'http://{manager_url}/api/conf/get', data={
        'project': project,
        'env': env,
        'ver': ver,
        'key': key,
    }).json().get('data')
    if res is None:
        res = default
    return res


def list_conf(manager_url=f'{WEB_HOST}:{WEB_PORT}', project=PROJECT_NAME, env=ENV, ver=VERSION, to_dict=True):
    res = requests.get(f'http://{manager_url}/api/conf/list', data={
        'project': project,
        'env': env,
        'ver': ver,
    }).json()
    if to_dict:
        result = {}
        for d in res:
            result[d["key"]] = getattr(builtins, d['value_type'])(d["value"])
        return result
    else:
        return res


def set_conf(key, value, manager_url=f'{WEB_HOST}:{WEB_PORT}', project=PROJECT_NAME, env=ENV, ver=VERSION):
    return requests.post(f'http://{manager_url}/api/conf/set', data={
        'project': project,
        'env': env,
        'ver': ver,
        'key': key,
        'value': value,
        'value_type': type(value).__name__,
    }).json()


if __name__ == '__main__':
    print(set_conf('es_port', 9200))
    print(list_conf())
    print(get_conf('es_port'))



from sqlalchemy import Boolean, Column, ForeignKey, Integer, String, Text, VARCHAR, BigInteger, DateTime
from uuid import UUID
from datetime import datetime, time, timedelta
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, Session
from conftab.default import SQLALCHEMY_DATABASE_URL


print(f"db路径: {SQLALCHEMY_DATABASE_URL}")
# 生成一个SQLAlchemy引擎
engine = create_engine(
    SQLALCHEMY_DATABASE_URL,
    # echo=True,
    connect_args={"check_same_thread": False},
)
# engine = create_engine(
#     SQLALCHEMY_DATABASE_URL,
#     pool_size=100,
#     pool_timeout=5,
#     pool_recycle=30,
#     max_overflow=0,
#     pool_pre_ping=True
# )
SessionLocal = sessionmaker(autocommit=False, autoflush=True, bind=engine)
Base = declarative_base()


def init_db():  # 初始化表
    Base.metadata.create_all(engine)


def drop_db():   # 删除表
    Base.metadata.drop_all(engine)


def get_db():
    db = SessionLocal()
    try:
        yield db
    finally:
        db.close()


class Mixin:
    def to_dict(self):
        return {c.name: getattr(self, c.name, None) for c in self.__table__.columns}


class Conf(Base, Mixin):
    __tablename__ = "conf"
    uuid = Column(String(512), primary_key=True, index=True)
    project = Column(String(64), index=True)
    env = Column(String(64), index=True)
    ver = Column(String(64), index=True)
    key = Column(String(64), index=True)
    value = Column(String(1024))
    value_type = Column(String(64), index=True)
    timecreate = Column(Integer, index=True)
    timeupdate = Column(Integer, index=True)
    time_create = Column(DateTime, index=True)
    time_update = Column(DateTime, index=True)


class Server(Base, Mixin):
    __tablename__ = "server"

    server_name = Column(String(32), primary_key=True, nullable=False, index=True, unique=True)
    server_type = Column(String(32), nullable=False, index=True)
    host = Column(String(64))
    port = Column(Integer)
    username = Column(String(64))
    password = Column(String(256))
    db = Column(String(256))





from conftab.function import get_conf, list_conf, set_conf
from conftab.default import WEB_HOST, WEB_PORT, PROJECT_NAME, ENV, VERSION


class Tab:
    def __init__(self, manager_url=f"{WEB_HOST}:{WEB_PORT}", project=PROJECT_NAME, env=ENV, ver=VERSION):
        if len(ms := manager_url.split(':')) < 2:
            self.host = ms[0]
            self.port = 7788
        else:
            self.host = ms[0]
            self.port = ms[-1]
        self.manager_url = f"{self.host}:{self.port}"
        self.project = project
        self.env = env
        self.ver = ver

    def get(self, key, default=None):
        res = get_conf(
            key, default=default, manager_url=self.manager_url, project=self.project, env=self.env, ver=self.ver)
        return res

    def list(self, to_dict=True):
        return list_conf(
            manager_url=self.manager_url, project=self.project, env=self.env, ver=self.ver, to_dict=to_dict)

    def set(self, key, value):
        return set_conf(key, value, manager_url=self.manager_url, project=self.project, env=self.env, ver=self.ver)



import click
import uvicorn
import socket
import os
from conftab.default import WEB_PORT, WEB_HOST


def run_app(host, port):
    from uvicorn.config import LOGGING_CONFIG
    from conftab.web import app

    # LOGGING_CONFIG['incremental'] = True
    # LOGGING_CONFIG['disable_existing_loggers'] = True
    # LOGGING_CONFIG["loggers"]["watchman"] = {"handlers": ["default"], "level": "INFO"}
    LOGGING_CONFIG["loggers"]["uvicorn"]["propagate"] = False
    LOGGING_CONFIG["formatters"]["default"][
        "fmt"] = '[%(levelprefix)s][%(asctime)s][%(name)s] "%(pathname)s:%(lineno)d" %(message)s'
    LOGGING_CONFIG["formatters"]["access"][
        "fmt"] = '[%(levelprefix)s][%(asctime)s][%(name)s] [%(client_addr)s] "%(request_line)s" %(status_code)s'
    uvicorn.run(app=app, host=host, port=port, workers=1, debug=True, log_config=LOGGING_CONFIG)


@click.group()
def ctl():
    pass


@click.command()
@click.option('-h', '--host', default="0.0.0.0", help='web管理端绑定host')
@click.option('-p', '--port', default=WEB_PORT, help='web管理端绑定端口号')
@click.option('-f', '--file', default="./conftab.db", help='web管理端绑定端口号')
def run(host, port, file):
    """web manager runner"""
    file_path = os.path.realpath(file)

    from conftab.default import set_url
    set_url(f'sqlite:///{file_path}')

    from conftab.model import init_db
    init_db()

    click.echo(f'start manager: {host}:{port} db路径: {file_path}')
    # sockobj = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sockobj = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sockobj.settimeout(3)
    # sockobj.bind((host, port))
    result = sockobj.connect_ex(('127.0.0.1', port))
    if result == 0:
        click.echo(result)
        raise SystemError('端口被占用了')
    run_app(host, port)


ctl.add_command(run)


if __name__ == '__main__':
    ctl()



WEB_HOST = '127.0.0.1'
WEB_PORT = 7788

PROJECT_NAME = 'default'

ENV = 'dev'

VERSION = '1.0.0'

# SQLALCHEMY_DATABASE_URL: str = 'sqlite:///:memory:'
SQLALCHEMY_DATABASE_URL: str = 'sqlite:///conftab.db'


def set_url(url):
    global SQLALCHEMY_DATABASE_URL
    SQLALCHEMY_DATABASE_URL = url
